package com.sun.java.swing.plaf.windows;

import java.awt.event.MouseEvent;
import java.awt.Dimension;
import javax.swing.Icon;
import javax.swing.event.MouseInputListener;
import java.awt.Rectangle;
import javax.swing.UIDefaults;
import java.awt.Color;
import java.awt.Graphics;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.MenuElement;
import javax.swing.ButtonModel;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.plaf.basic.BasicMenuUI;

public class WindowsMenuUI extends BasicMenuUI
{
    protected Integer menuBarHeight;
    protected boolean hotTrackingOn;
    final WindowsMenuItemUIAccessor accessor;
    
    public WindowsMenuUI() {
        this.accessor = new WindowsMenuItemUIAccessor() {
            @Override
            public JMenuItem getMenuItem() {
                return WindowsMenuUI.this.menuItem;
            }
            
            @Override
            public TMSchema.State getState(JMenuItem menuItem) {
                TMSchema.State state = menuItem.isEnabled() ? TMSchema.State.NORMAL : TMSchema.State.DISABLED;
                ButtonModel model = menuItem.getModel();
                if (model.isArmed() || model.isSelected()) {
                    state = (menuItem.isEnabled() ? TMSchema.State.PUSHED : TMSchema.State.DISABLEDPUSHED);
                }
                else if (model.isRollover() && ((JMenu)menuItem).isTopLevelMenu()) {
                    state = (menuItem.isEnabled() ? TMSchema.State.HOT : TMSchema.State.DISABLEDHOT);
                    MenuElement[] subElements = ((JMenuBar)menuItem.getParent()).getSubElements();
                    int length = subElements.length;
                    int i = 0;
                    while (i < length) {
                        if (((JMenuItem)subElements[i]).isSelected()) {
                            break;
                        }
                        ++i;
                    }
                }
                if (!((JMenu)menuItem).isTopLevelMenu()) {
                    if (state == TMSchema.State.PUSHED) {
                        state = TMSchema.State.HOT;
                    }
                    else if (state == TMSchema.State.DISABLEDPUSHED) {
                        state = TMSchema.State.DISABLEDHOT;
                    }
                }
                if (((JMenu)menuItem).isTopLevelMenu() && WindowsMenuItemUI.isVistaPainting() && !WindowsMenuBarUI.isActive(menuItem)) {
                    state = TMSchema.State.DISABLED;
                    return state;
                }
                return state;
            }
            
            @Override
            public TMSchema.Part getPart(JMenuItem menuItem) {
                return ((JMenu)menuItem).isTopLevelMenu() ? TMSchema.Part.MP_BARITEM : TMSchema.Part.MP_POPUPITEM;
            }
        };
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new WindowsMenuUI();
    }
    
    @Override
    protected void installDefaults() {
        super.installDefaults();
        if (!WindowsLookAndFeel.isClassicWindows()) {
            this.menuItem.setRolloverEnabled(true);
        }
        this.menuBarHeight = Integer.valueOf(UIManager.getInt("MenuBar.height"));
        Object value = UIManager.get("MenuBar.rolloverEnabled");
        this.hotTrackingOn = (!(value instanceof Boolean) || (boolean)value);
    }
    
    @Override
    protected void paintBackground(Graphics g, JMenuItem menuItem, Color bgColor) {
        if (WindowsMenuItemUI.isVistaPainting()) {
            WindowsMenuItemUI.paintBackground(this.accessor, g, menuItem, bgColor);
            return;
        }
        JMenu menuItem2 = (JMenu)menuItem;
        ButtonModel model = menuItem2.getModel();
        if (WindowsLookAndFeel.isClassicWindows() || !menuItem2.isTopLevelMenu() || (XPStyle.getXP() != null && (model.isArmed() || model.isSelected()))) {
            super.paintBackground(g, menuItem2, bgColor);
            return;
        }
        Color color = g.getColor();
        int width = menuItem2.getWidth();
        int height = menuItem2.getHeight();
        UIDefaults lookAndFeelDefaults = UIManager.getLookAndFeelDefaults();
        Color color2 = lookAndFeelDefaults.getColor("controlLtHighlight");
        Color color3 = lookAndFeelDefaults.getColor("controlShadow");
        g.setColor(menuItem2.getBackground());
        g.fillRect(0, 0, width, height);
        if (menuItem2.isOpaque()) {
            if (model.isArmed() || model.isSelected()) {
                g.setColor(color3);
                g.drawLine(0, 0, width - 1, 0);
                g.drawLine(0, 0, 0, height - 2);
                g.setColor(color2);
                g.drawLine(width - 1, 0, width - 1, height - 2);
                g.drawLine(0, height - 2, width - 1, height - 2);
            }
            else if (model.isRollover() && model.isEnabled()) {
                boolean n = false;
                MenuElement[] subElements = ((JMenuBar)menuItem2.getParent()).getSubElements();
                int i = 0;
                while (i < subElements.length) {
                    if (((JMenuItem)subElements[i]).isSelected()) {
                        n = true;
                        break;
                    }
                    ++i;
                }
                if (n == 0) {
                    if (XPStyle.getXP() != null) {
                        g.setColor(this.selectionBackground);
                        g.fillRect(0, 0, width, height);
                    }
                    else {
                        g.setColor(color2);
                        g.drawLine(0, 0, width - 1, 0);
                        g.drawLine(0, 0, 0, height - 2);
                        g.setColor(color3);
                        g.drawLine(width - 1, 0, width - 1, height - 2);
                        g.drawLine(0, height - 2, width - 1, height - 2);
                    }
                }
            }
        }
        g.setColor(color);
        return;
    }
    
    @Override
    protected void paintText(Graphics graphics, JMenuItem menuItem, Rectangle rectangle, String s) {
        if (WindowsMenuItemUI.isVistaPainting()) {
            WindowsMenuItemUI.paintText(this.accessor, graphics, menuItem, rectangle, s);
            return;
        }
        JMenu menu = (JMenu)menuItem;
        ButtonModel model = menuItem.getModel();
        Color color = graphics.getColor();
        int rollover = model.isRollover();
        if (rollover != 0 && menu.isTopLevelMenu()) {
            MenuElement[] subElements = ((JMenuBar)menu.getParent()).getSubElements();
            int i = 0;
            while (i < subElements.length) {
                if (((JMenuItem)subElements[i]).isSelected()) {
                    rollover = 0;
                    break;
                }
                ++i;
            }
        }
        if ((model.isSelected() && (WindowsLookAndFeel.isClassicWindows() || !menu.isTopLevelMenu())) || (XPStyle.getXP() != null && (rollover != 0 || model.isArmed() || model.isSelected()))) {
            graphics.setColor(this.selectionForeground);
        }
        WindowsGraphicsUtils.paintText(graphics, menuItem, rectangle, s, 0);
        graphics.setColor(color);
        return;
    }
    
    @Override
    protected MouseInputListener createMouseInputListener(JComponent component) {
        return new WindowsMenuUI.WindowsMouseInputHandler();
    }
    
    @Override
    protected Dimension getPreferredMenuItemSize(JComponent c, Icon checkIcon, Icon arrowIcon, int defaultTextIconGap) {
        Dimension preferredMenuItemSize = super.getPreferredMenuItemSize(c, checkIcon, arrowIcon, defaultTextIconGap);
        if (c instanceof JMenu && ((JMenu)c).isTopLevelMenu() && this.menuBarHeight != null && preferredMenuItemSize.height < (int)this.menuBarHeight) {
            preferredMenuItemSize.height = (int)this.menuBarHeight;
            return preferredMenuItemSize;
        }
        return preferredMenuItemSize;
    }
    
    protected class WindowsMouseInputHandler extends BasicMenuUI.MouseInputHandler
    {
        @Override
        public void mouseEntered(MouseEvent e) {
            super.mouseEntered(e);
            JMenu menu = (JMenu)e.getSource();
            if (WindowsMenuUI.this.hotTrackingOn && menu.isTopLevelMenu() && menu.isRolloverEnabled()) {
                menu.getModel().setRollover(true);
                WindowsMenuUI.this.menuItem.repaint();
            }
        }
        
        @Override
        public void mouseExited(MouseEvent e) {
            super.mouseExited(e);
            JMenu menu = (JMenu)e.getSource();
            ButtonModel model = menu.getModel();
            if (menu.isRolloverEnabled()) {
                model.setRollover(false);
                WindowsMenuUI.this.menuItem.repaint();
            }
        }
    }
}
